@@ -5,6 +5,7 @@ from .sql import SQLQueue
 from .logging import LogHandle
 from exchangelib import Credentials as Exch_Cred, Configuration, DELEGATE
 from exchangelib.errors import UnauthorizedError, TransportError
+from traceback import format_exc
 
 import pathlib as pl
 
@@ -258,28 +259,37 @@ class Toolbox(TBoxBase, SQLQueue, LogHandle):
 
             return engine
 
-    def default_exchange_conn(self):
+    def default_exchange_conn(self, user_name=None, user_pass=None, auto_renew=False):
         """
         Create default Exchangelib e-mail connection
 
         :return: Returns Exchange instance class, which is a child class of Exchangelib's Account instance class
         """
-        def email_check(mc):
+        def email_check(mc, un, up):
             from . import EmailGUI
             from .data import DataConfig
+            from .data import CryptHandle
 
-            if not isinstance(mc, DataConfig):
-                raise ValueError("'main_config' %r is not an instance of DataConfig" % mc)
+            if un and up:
+                if not isinstance(un, CryptHandle):
+                    raise ValueError("'user_name' %r is not an instance of CryptHandle" % un)
+                if not isinstance(up, CryptHandle):
+                    raise ValueError("'user_pass' %r is not an instance of CryptHandle" % up)
 
-            if 'Exchange_Server' not in mc.keys() or 'Exchange_Email' not in mc.keys():
-                s = EmailGUI()
+                return True
+            elif mc:
+                if not isinstance(mc, DataConfig):
+                    raise ValueError("'main_config' %r is not an instance of DataConfig" % mc)
 
-                if not s.server.decrypt() or not s.email_addr.decrypt():
-                    return False
+                if 'Exchange_Server' not in mc.keys() or 'Exchange_Email' not in mc.keys():
+                    s = EmailGUI()
 
-                mc['Exchange_Server'] = s.server
-                mc['Exchange_Email'] = s.email_addr
-                mc.sync()
+                    if not s.server.decrypt() or not s.email_addr.decrypt():
+                        return False
+
+                    mc['Exchange_Server'] = s.server
+                    mc['Exchange_Email'] = s.email_addr
+                    mc.sync()
 
             return True
 
@@ -297,59 +307,83 @@ class Toolbox(TBoxBase, SQLQueue, LogHandle):
 
             return True
 
-        def cred_check(mc):
+        def cred_check(mc, un, up):
             from . import CredentialsGUI
             from .data import DataConfig
 
-            if not isinstance(mc, DataConfig):
-                raise ValueError("'main_config' %r is not an instance of DataConfig" % mc)
+            if un and up:
+                if not un.decrypt():
+                    raise ValueError("'user_name' is an empty CryptHandle object")
+                elif not up.decrypt():
+                    raise ValueError("'user_pass' is an empty CryptHandle object")
 
-            if 'Exchange_Cred' not in mc.keys():
-                s = CredentialsGUI()
+                return [un, up]
+            elif mc:
+                if not isinstance(mc, DataConfig):
+                    raise ValueError("'main_config' %r is not an instance of DataConfig" % mc)
 
-                if not s.credentials.username.decrypt() or not s.credentials.password.decrypt():
-                    return False
+                if 'Exchange_Cred' not in mc.keys():
+                    s = CredentialsGUI()
 
-                mc['Exchange_Cred'] = s.credentials
-                mc.sync()
+                    if not s.credentials.username.decrypt():
+                        raise ValueError("'user_name' is an empty CryptHandle object")
+                    elif not s.credentials.password.decrypt():
+                        raise ValueError("'user_pass' is an empty CryptHandle object")
 
-            return True
+                    mc['Exchange_Cred'] = s.credentials
+                    mc.sync()
 
-        def cred_change(mc):
+                return [mc['Exchange_Cred'].username, mc['Exchange_Cred'].password]
+            else:
+                return [None, None]
+
+        def cred_change(mc, un, up):
             from . import CredentialsGUI
 
             s = CredentialsGUI(cred=mc['Exchange_Cred'])
 
-            if not s.credentials.username.decrypt() or not s.credentials.password.decrypt():
-                return False
+            if not s.credentials.username.decrypt():
+                raise ValueError("'user_name' is an empty CryptHandle object")
+            elif not s.credentials.password.decrypt():
+                raise ValueError("'user_pass' is an empty CryptHandle object")
 
-            mc['Exchange_Cred'] = s.credentials
-            mc.sync()
-
-            return True
+            if un and up:
+                return [True, s.credentials.username, s.credentials.password]
+            elif mc:
+                mc['Exchange_Cred'] = s.credentials
+                mc.sync()
+                return [True, None, None]
+            else:
+                return [False, None, None]
 
         from .exchangelib import Exchange
 
-        if email_check(self.main_config) and cred_check(self.main_config):
-            try:
-                cred = Exch_Cred(self.main_config['Exchange_Cred'].username.decrypt(),
-                                 self.main_config['Exchange_Cred'].password.decrypt())
-                config = Configuration(server=self.main_config['Exchange_Server'].decrypt(), credentials=cred)
-                return Exchange(primary_smtp_address=self.main_config['Exchange_Email'].decrypt(), config=config,
-                                autodiscover=False, access_type=DELEGATE)
-            except UnauthorizedError as e:
-                print("Error Code {0}, {1}".format(type(e).__name__, str(e)))
+        if email_check(self.main_config, user_name, user_pass):
+            uname, upass = cred_check(self.main_config, user_name, user_pass)
 
-                if cred_change(self.main_config):
-                    self.default_exchange_conn()
+            if uname and upass:
+                try:
+                    cred = Exch_Cred(uname.decrypt(), upass.decrypt())
+                    config = Configuration(server=self.main_config['Exchange_Server'].decrypt(), credentials=cred)
+                    return Exchange(primary_smtp_address=self.main_config['Exchange_Email'].decrypt(), config=config,
+                                    autodiscover=False, access_type=DELEGATE, auto_renew=auto_renew)
+                except UnauthorizedError as e:
+                    self.write_to_log(format_exc())
+                    self.write_to_log("Error Code {0}, {1}".format(type(e).__name__, str(e)))
+                    cred_chg, uname, upass = cred_change(self.main_config, user_name, user_pass)
 
-            except (ValueError, TransportError) as e:
-                print("Error Code {0}, {1}".format(type(e).__name__, str(e)))
+                    if cred_chg:
+                        self.default_exchange_conn(user_name=uname, user_pass=upass, auto_renew=auto_renew)
 
-                if email_change(self.main_config):
-                    self.default_exchange_conn()
-            except Exception as e:
-                print("Error Code {0}, {1}".format(type(e).__name__, str(e)))
+                except (ValueError, TransportError) as e:
+                    self.write_to_log(format_exc())
+                    self.write_to_log("Error Code {0}, {1}".format(type(e).__name__, str(e)))
+
+                    if email_change(self.main_config):
+                        self.default_exchange_conn(user_name=user_name, user_pass=user_pass, auto_renew=auto_renew)
+                except Exception as e:
+                    self.write_to_log(format_exc())
+                    self.write_to_log("Error Code {0}, {1}".format(type(e).__name__, str(e)))
 
     def __find_engine(self, config):
         from .sql import SQLConfig, SQLEngineClass
